@@ -86,7 +86,6 @@ const ModuleVersions LoaderVersionInfo = {
     ABI_VIDEODRV_VERSION,
     ABI_XINPUT_VERSION,
     ABI_EXTENSION_VERSION,
-    ABI_FONT_VERSION
 };
 
 static int ModuleDuplicated[] = { };
@@ -142,7 +141,7 @@ InitPathList(const char *path)
             if (addslash)
                 len++;
             save = list;
-            list = realloc(list, (n + 2) * sizeof(char *));
+            list = reallocarray(list, n + 2, sizeof(char *));
             if (!list) {
                 if (save) {
                     save[n] = NULL;
@@ -244,7 +243,7 @@ InitPatterns(const char **patternlist)
         for (i = 0, s = patternlist; *s; i++, s++)
             if (*s == DEFAULT_LIST)
                 i += sizeof(stdPatterns) / sizeof(stdPatterns[0]) - 1 - 1;
-        patterns = malloc((i + 1) * sizeof(PatternRec));
+        patterns = xallocarray(i + 1, sizeof(PatternRec));
         if (!patterns) {
             return NULL;
         }
@@ -323,7 +322,7 @@ InitSubdirs(const char **subdirlist)
                 }
             }
         }
-        subdirs = malloc((i * 2 + 1) * sizeof(char *));
+        subdirs = xallocarray(i * 2 + 1, sizeof(char *));
         if (!subdirs) {
             free(tmp_subdirlist);
             return NULL;
@@ -530,8 +529,8 @@ LoaderListDirs(const char **subdirlist, const char **patternlist)
                             match[1].rm_so != -1) {
                             len = match[1].rm_eo - match[1].rm_so;
                             save = listing;
-                            listing = realloc(listing,
-                                              (n + 2) * sizeof(char *));
+                            listing = reallocarray(listing, n + 2,
+                                                   sizeof(char *));
                             if (!listing) {
                                 if (save) {
                                     save[n] = NULL;
@@ -613,8 +612,6 @@ CheckVersion(const char *module, XF86ModuleVersionInfo * data,
             ver = LoaderVersionInfo.xinputVersion;
         else if (!strcmp(data->abiclass, ABI_CLASS_EXTENSION))
             ver = LoaderVersionInfo.extensionVersion;
-        else if (!strcmp(data->abiclass, ABI_CLASS_FONT))
-            ver = LoaderVersionInfo.fontVersion;
 
         abimaj = GET_ABI_MAJOR(data->abiversion);
         abimin = GET_ABI_MINOR(data->abiversion);
@@ -629,9 +626,9 @@ CheckVersion(const char *module, XF86ModuleVersionInfo * data,
                 else
                     errtype = X_ERROR;
                 xf86MsgVerb(errtype, 0,
-                            "module ABI major version (%d) doesn't"
+                            "%s: module ABI major version (%d) doesn't"
                             " match the server's version (%d)\n",
-                            abimaj, vermaj);
+                            module, abimaj, vermaj);
                 if (!(LoaderOptions & LDR_OPT_ABI_MISMATCH_NONFATAL))
                     return FALSE;
             }
@@ -641,9 +638,9 @@ CheckVersion(const char *module, XF86ModuleVersionInfo * data,
                 else
                     errtype = X_ERROR;
                 xf86MsgVerb(errtype, 0,
-                            "module ABI minor version (%d) is "
+                            "%s: module ABI minor version (%d) is "
                             "newer than the server's version "
-                            "(%d)\n", abimin, vermin);
+                            "(%d)\n", module, abimin, vermin);
                 if (!(LoaderOptions & LDR_OPT_ABI_MISMATCH_NONFATAL))
                     return FALSE;
             }
@@ -654,24 +651,24 @@ CheckVersion(const char *module, XF86ModuleVersionInfo * data,
     if (req) {
         if (req->majorversion != MAJOR_UNSPEC) {
             if (data->majorversion != req->majorversion) {
-                xf86MsgVerb(X_WARNING, 2, "module major version (%d) "
+                xf86MsgVerb(X_WARNING, 2, "%s: module major version (%d) "
                             "doesn't match required major version (%d)\n",
-                            data->majorversion, req->majorversion);
+                            module, data->majorversion, req->majorversion);
                 return FALSE;
             }
             else if (req->minorversion != MINOR_UNSPEC) {
                 if (data->minorversion < req->minorversion) {
-                    xf86MsgVerb(X_WARNING, 2, "module minor version (%d) "
+                    xf86MsgVerb(X_WARNING, 2, "%s: module minor version (%d) "
                                 "is less than the required minor version (%d)\n",
-                                data->minorversion, req->minorversion);
+                                module, data->minorversion, req->minorversion);
                     return FALSE;
                 }
                 else if (data->minorversion == req->minorversion &&
                          req->patchlevel != PATCH_UNSPEC) {
                     if (data->patchlevel < req->patchlevel) {
-                        xf86MsgVerb(X_WARNING, 2, "module patch level (%d) "
+                        xf86MsgVerb(X_WARNING, 2, "%s: module patch level (%d) "
                                     "is less than the required patch level (%d)\n",
-                                    data->patchlevel, req->patchlevel);
+                                    module, data->patchlevel, req->patchlevel);
                         return FALSE;
                     }
                 }
@@ -680,8 +677,9 @@ CheckVersion(const char *module, XF86ModuleVersionInfo * data,
         if (req->moduleclass) {
             if (!data->moduleclass ||
                 strcmp(req->moduleclass, data->moduleclass)) {
-                xf86MsgVerb(X_WARNING, 2, "Module class (%s) doesn't match "
+                xf86MsgVerb(X_WARNING, 2, "%s: Module class (%s) doesn't match "
                             "the required class (%s)\n",
+                            module,
                             data->moduleclass ? data->moduleclass : "<NONE>",
                             req->moduleclass);
                 return FALSE;
@@ -689,8 +687,9 @@ CheckVersion(const char *module, XF86ModuleVersionInfo * data,
         }
         else if (req->abiclass != ABI_CLASS_NONE) {
             if (!data->abiclass || strcmp(req->abiclass, data->abiclass)) {
-                xf86MsgVerb(X_WARNING, 2, "ABI class (%s) doesn't match the "
+                xf86MsgVerb(X_WARNING, 2, "%s: ABI class (%s) doesn't match the "
                             "required ABI class (%s)\n",
+                            module,
                             data->abiclass ? data->abiclass : "<NONE>",
                             req->abiclass);
                 return FALSE;
@@ -705,15 +704,16 @@ CheckVersion(const char *module, XF86ModuleVersionInfo * data,
             maj = GET_ABI_MAJOR(data->abiversion);
             min = GET_ABI_MINOR(data->abiversion);
             if (maj != reqmaj) {
-                xf86MsgVerb(X_WARNING, 2, "ABI major version (%d) doesn't "
+                xf86MsgVerb(X_WARNING, 2, "%s: ABI major version (%d) doesn't "
                             "match the required ABI major version (%d)\n",
-                            maj, reqmaj);
+                            module, maj, reqmaj);
                 return FALSE;
             }
             /* XXX Maybe this should be the other way around? */
             if (min > reqmin) {
-                xf86MsgVerb(X_WARNING, 2, "module ABI minor version (%d) "
-                            "is newer than that available (%d)\n", min, reqmin);
+                xf86MsgVerb(X_WARNING, 2, "%s: module ABI minor version (%d) "
+                            "is newer than that available (%d)\n",
+                            module, min, reqmin);
                 return FALSE;
             }
         }
@@ -918,7 +918,7 @@ LoadModule(const char *module, const char *path, const char **subdirlist,
         goto LoadModule_fail;
     }
 
-    /* 
+    /*
      * if the module name is not a full pathname, we need to
      * check the elements in the path
      */
@@ -938,7 +938,7 @@ LoadModule(const char *module, const char *path, const char **subdirlist,
         }
     }
 
-    /* 
+    /*
      * did we find the module?
      */
     if (!found) {
